package de.ozgcloud.eingang.semantik.formbased.mantelantrag;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.test.context.ActiveProfiles;

import de.ozgcloud.common.binaryfile.TempFileUtils;
import de.ozgcloud.common.test.TestUtils;
import de.ozgcloud.eingang.Application;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFileTestFactory;
import de.ozgcloud.eingang.router.VorgangService;
import de.ozgcloud.eingang.semantik.SemantikAdapter;
import de.ozgcloud.eingang.semantik.enginebased.formsolutions.FormSolutionsEngineBasedAdapterITCase;
import lombok.SneakyThrows;

@ActiveProfiles({ "local", "itcase" })
@SpringBootTest(classes = Application.class, properties = { "ozgcloud.xta.identifier=afmsh:ozg-cloud-utopia-test" })
public class MantelantragITCase {

	private static final String FILE_NAME_XDOMEA = "mantelantrag/4620-EH6C_b3c9168a-6ae9-4361-8b2f-6837bb341021_Geschaeftsgang.Geschaeftsgang.0201.xml";
	private static final String FILE_NAME_MANTELANTRAG = "mantelantrag/4620-EH6C_7d703670-15b6-42b2-8cd1-88a7e4c494b9_Antrag_Max_Mustermann_SGBXII.xml";

	@MockBean
	VorgangService vorgangService;

	@SpyBean
	SemantikAdapter semantikAdapter;

	@Nested
	class TestZustaendigeStelle {

		@Captor
		ArgumentCaptor<FormData> formDataCaptor;

		@Test
		void zustaendigeStelleExists() {
			var formData = prepareTestData();
			doReturn("test").when(vorgangService).createVorgang(any(FormData.class));
			semantikAdapter.processFormData(formData);

			verify(vorgangService).createVorgang(formDataCaptor.capture());
			FormData capturedFormData = formDataCaptor.getValue();
			assertThat(capturedFormData).isNotNull();
		}

		@Test
		void zustaendigeStelleIsUtopia() {
			var formData = prepareTestData();
			doReturn("test").when(vorgangService).createVorgang(any(FormData.class));
			semantikAdapter.processFormData(formData);

			verify(vorgangService).createVorgang(formDataCaptor.capture());
			FormData capturedFormData = formDataCaptor.getValue();

			assertThat(capturedFormData.getZustaendigeStelle().getOrganisationseinheitenId()).isEqualTo("123");
		}

	}

	private FormData prepareTestData() {
		var xdomeaFile = IncomingFileTestFactory.createBuilder()
				.name(FILE_NAME_XDOMEA)
				.file(asFile(FILE_NAME_XDOMEA))
				.size(getFileSize(FILE_NAME_XDOMEA))
				.contentType("application/xml")
				.build();
		var mantelantragFile = IncomingFileTestFactory.createBuilder()
				.name(FILE_NAME_MANTELANTRAG)
				.file(asFile(FILE_NAME_MANTELANTRAG))
				.size(getFileSize(FILE_NAME_MANTELANTRAG))
				.contentType("application/xml")
				.build();

		return FormData.builder().representations(List.of(xdomeaFile, mantelantragFile)).build();
	}

	@SneakyThrows
	private long getFileSize(String fileName) {
		Path filePath = Path.of(FormSolutionsEngineBasedAdapterITCase.class.getClassLoader().getResource(fileName).toURI());
		return Files.size(filePath);
	}

	private File asFile(String path) {
		return TempFileUtils.writeTmpFile(TestUtils.loadFile(path));
	}
}
