package de.ozgcloud.eingang.xdomea;

import static de.ozgcloud.eingang.xdomea.XMLHelper.*;

import java.util.List;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;

import org.springframework.stereotype.Component;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.ozgcloud.eingang.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.IncomingFile;

@Component
public class XdomeaXMLValueReader {

	private static final String DATEINAME_NODE_QUERY_STRING = "//Hauptobjekt//Dateiname";
	private static final XPathExpression DATEINAME_NODE_QUERY = compileXPathExpression(DATEINAME_NODE_QUERY_STRING);

	public List<String> readRepresentationFileNames(IncomingFile xdomeaXMLFile) {
		return getTextsFromNodes(
				queryDateinameNodeList(
						XMLHelper.parseDocument(xdomeaXMLFile)
								.getDocumentElement()
				)
		);
	}

	private NodeList queryDateinameNodeList(Element contextElement) {
		try {
			return (NodeList) DATEINAME_NODE_QUERY
					.evaluate(
							contextElement,
							XPathConstants.NODESET
					);
		} catch (XPathExpressionException e) {
			throw new TechnicalException("Failed to execute xpath search!", e);
		}
	}

	private List<String> getTextsFromNodes(NodeList nodeList) {
		return streamNodeList(nodeList)
				.map(Node::getTextContent)
				.map(String::trim)
				.toList();
	}

	private Stream<Node> streamNodeList(NodeList nodeList) {
		return IntStream.range(0, nodeList.getLength())
				.mapToObj(nodeList::item);
	}

}
